package info.textgrid.middleware;

import info.textgrid.middleware.oaipmh.ElementType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.OaiDcType;
import info.textgrid.middleware.oaipmh.ObjectFactory;


import java.util.List;


import javax.xml.bind.JAXBElement;

/**
 * The DublinCoreBuilder is responsible for generating a List of JAXBElements in DublinCore standard. 
 * Each function get a list of strings representing the mapped textgrid metadata for the specific DC-field   
 * 
 * @author Maximilian Brodhun SUB
 * @version 1.1
 * @since 18.02.2014
 * 
 */

public final class DublinCoreBuilder {
	
	private OaiDcType tgMappedDC = new OaiDcType();
	private ObjectFactory oaiDcObj = new ObjectFactory();
	private MetadataType metadata = new MetadataType();
	
	/**
	 * Constructor to build an DublinCore element for an OAIPMH request by getting textGrid metadata
	 * @param tgMappedDC
	 * @param oaiDcObj2
	 */
	
	public DublinCoreBuilder(OaiDcType tgMappedDC, ObjectFactory oaiDcObj2){
		
		this.tgMappedDC = tgMappedDC;
		this.oaiDcObj = oaiDcObj2;
				
	}
    
	/**
	 * Takes a Stringlist containing the contributors and build the specific dublinCore element for each 
	 * @param contributors
	 */
	
	public void setContributor(List<String> contributors){
		
		for (String dccontributor : contributors) {
	      	
			ElementType contributorElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreContributor = getOaiDcObj().createContributor(contributorElement);
	    	contributorElement.setValue(dccontributor);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreContributor);
    	}
	}
	
	/**
	 * Takes a Stringlist containing the creators and build the specific dublinCore element for each 
	 * @param creators
	 */
	public void setCreator(List<String> creators){
		
		for (String dccreator : creators) {
	      	
			ElementType creatorElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreCreator = getOaiDcObj().createCreator(creatorElement);
	    	creatorElement.setValue(dccreator);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreCreator);
				    	   	
	    }

	}
	
	/**
	 * Takes a Stringlist containing the coverages and build the specific dublinCore element for each 
	 * @param coverages
	 */
	public void setCoverage(List<String> coverages){
	    for (String dccoverage : coverages) {
	      	ElementType coverageElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreCoverage = getOaiDcObj().createCoverage(coverageElement);
	     	coverageElement.setValue(dccoverage);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreCoverage);
	    }
			
	}
	
	/**
	 * Takes a Stringlist containing the dates and build the specific dublinCore element for each 
	 * @param dates
	 */
	public void setDate(List<String> dates){
	    for (String dcdate : dates) {
	      	ElementType dateElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreDate = getOaiDcObj().createDate(dateElement);
	    	dateElement.setValue(dcdate);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreDate);
	    }
		
		
	}
	
	/**
	 * Takes a Stringlist containing the descriptions and build the specific dublinCore element for each 
	 * @param descriptions
	 */
	public void setDescription(List<String> descriptions){
	    for (String dcdescription : descriptions) {
	      	ElementType descriptionElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreDescription = getOaiDcObj().createDescription(descriptionElement);
	    	descriptionElement.setValue(dcdescription);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreDescription);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the formats and build the specific dublinCore element for each 
	 * @param formats
	 */
	public void setFormat(List<String> formats){
	    for (String dcformat : formats) {
	      	ElementType formatElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreFormat = getOaiDcObj().createFormat(formatElement);
	    	formatElement.setValue(dcformat);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreFormat);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the identifiers and build the specific dublinCore element for each 
	 * @param identifiers
	 */
	public void setIdentifier(List<String> identifiers){
	    for (String dcidentifier : identifiers) {
	      	ElementType identifierElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreIdentifier = getOaiDcObj().createIdentifier(identifierElement);
	    	identifierElement.setValue(dcidentifier);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreIdentifier);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the languages and build the specific dublinCore element for each 
	 * @param languages
	 */
	public void setLanguage(List<String> languages){
	    for (String dclanguage : languages) {
	      	ElementType languageElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreLanguage = getOaiDcObj().createLanguage(languageElement);
	    	languageElement.setValue(dclanguage);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreLanguage);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the publishers and build the specific dublinCore element for each 
	 * @param publishers
	 */
	public void setPublisher(List<String> publishers){
	    for (String dcpublisher : publishers) {
	      	ElementType publisherElement = new ElementType();
	    	JAXBElement<ElementType> dcCorePublisher = getOaiDcObj().createPublisher(publisherElement);
	    	publisherElement.setValue(dcpublisher);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCorePublisher);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the relations and build the specific dublinCore element for each 
	 * @param relations
	 */
	public void setRelation(List<String> relations){
	    for (String dcrelation : relations) {
	      	ElementType relationElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreRelation = getOaiDcObj().createRelation(relationElement);
	    	relationElement.setValue(dcrelation);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreRelation);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the rights and build the specific dublinCore element for each 
	 * @param rights
	 */
	public void setRights(List<String> rights){
	    for (String dcrights : rights) {
	      	ElementType rightsElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreRights = getOaiDcObj().createRights(rightsElement);
	    	rightsElement.setValue(dcrights);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreRights);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the sources and build the specific dublinCore element for each 
	 * @param sources
	 */
	public void setSource(List<String> sources){
	    for (String dcsource : sources) {
	      	ElementType sourcesElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreSource = getOaiDcObj().createSource(sourcesElement);
	    	sourcesElement.setValue(dcsource);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreSource);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the subjects and build the specific dublinCore element for each 
	 * @param subjects
	 */
	public void setSubject(List<String> subjects){
	    for (String dcsubject : subjects) {
	      	ElementType subjectElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreSubject = getOaiDcObj().createSubject(subjectElement);
	    	subjectElement.setValue(dcsubject);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreSubject);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the titles and build the specific dublinCore element for each 
	 * @param titles
	 */
	public void setTitle(List<String> titles){
	    for (String dctitle : titles) {
	      	ElementType titlesElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreTitle = getOaiDcObj().createTitle(titlesElement);
	    	titlesElement.setValue(dctitle);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreTitle);
	    }
		
	}
	
	/**
	 * Takes a Stringlist containing the types and build the specific dublinCore element for each 
	 * @param types
	 */
	public void setType(List<String> types){
	    for (String dctype : types) {
	      	ElementType typeElement = new ElementType();
	    	JAXBElement<ElementType> dcCoreType = getOaiDcObj().createType(typeElement);
	    	typeElement.setValue(dctype);
	    	getTgMappedDC().getTitleOrCreatorOrSubject().add(dcCoreType);
	    }
		
	}
	
	/**
	 * fetching dublinCore element and putting it into the metadata element for the OAIPMH record response
	 * @return metadata element with the dublinCore values for a specific object
	 */
	public MetadataType getDC(){
		metadata.setAny(getOaiDcObj().createDc(getTgMappedDC()));
		return metadata;
	}

	/**
	 * 
	 * @return tgMappedDc
	 */
	public OaiDcType getTgMappedDC() {
		return tgMappedDC;
	}

	/**
	 * 
	 * @return oaiDcObj
	 */
	public ObjectFactory getOaiDcObj() {
		return oaiDcObj;
	}


}
