package info.textgrid.middleware;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBElement;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.oaipmh.DeletedRecordType;
import info.textgrid.middleware.oaipmh.GranularityType;
import info.textgrid.middleware.oaipmh.OAIPMHType;
import info.textgrid.namespaces.middleware.tgcrud.common.TextGridMimetypes;

/**
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 *
 */

public class OaiPmhTest {

  public static OAI_ESClient oaiEsClient;  
  DeletedRecordType bla = DeletedRecordType.NO;
  GranularityType bla2 = GranularityType.YYYY_MM_DD_THH_MM_SS_Z;
  private RepIdentification rep = new RepIdentification("TextGrid-Repository", "www.textgridrep.de",
      this.bla, "2011-06-11T02:32:40Z", this.bla2, "2.0", "textgrid-support@gwdg.de");

  private static RecordDelivererDC record;
  private static RecordDelivererIDIOM recordIDIOM;
  private static RecordListDelivererDC recordList;
  private static RecordListDelivererIDIOM recordListIDIOM;
  private static IdentifierListDelivererDC identifierList = new IdentifierListDelivererDC(true, false);
  private static IdentifierListDelivererIDIOM identifierListIDIOM = new IdentifierListDelivererIDIOM(true, false);
  private MetadataFormatListDeliverer metadataFormatList = new MetadataFormatListDeliverer();
  private SetDeliverer setListDARIAH = new SetDeliverer(false, true);
  private SetDeliverer setListTextGrid = new SetDeliverer(true, false);
  
  private OAIPMHImpl request = new OAIPMHImpl(
		  this.rep, OaiPmhTest.record, OaiPmhTest.recordIDIOM, OaiPmhTest.recordList,
		  OaiPmhTest.recordListIDIOM, this.metadataFormatList,
		  this.setListDARIAH, OaiPmhTest.identifierList, OaiPmhTest.identifierListIDIOM);

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUp() throws Exception {
    oaiEsClient = new OAI_ESClient("localhost", "9302", "tg-esx2-instance");
    OAI_ESClient.setEsIndex("textgrid-public");
    OAI_ESClient.setEsType("metadata");
    
    record = new RecordDelivererDC(true, false);
    record.setOaiEsClient(oaiEsClient);
    record.setWorkFields(TGConstants.TGWorkFields);
    
    recordIDIOM = new RecordDelivererIDIOM(true, false);
    recordIDIOM.setOaiEsClient(oaiEsClient);   
    
    recordList = new RecordListDelivererDC(true, false);
    recordList.setFields(TGConstants.TGFields);
    recordList.setWorkFields(TGConstants.TGWorkFields);
    recordList.setOaiEsClient(oaiEsClient);
    recordList.setSearchResponseSize("100");
    
    recordListIDIOM = new RecordListDelivererIDIOM(true, false);
    recordListIDIOM.setOaiEsClient(oaiEsClient);   
    
    identifierList = new IdentifierListDelivererDC(true, false);
    identifierList.setOaiEsClient(oaiEsClient);
    identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
  }

  /**
   * @throws DatatypeConfigurationException
   */
  @Test
  @Ignore
  public void testDateNow() throws DatatypeConfigurationException {
    System.out.println("---------------Test Datestamp Parsing with now date -----------------");
    XMLGregorianCalendar nowTest = OAIPMHUtillities.getXMLGregorianCalendarNow();
    System.out.println(nowTest);
    System.out.println("-------------------------------------------");
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  @Ignore
  public void testDateGregorianType() throws ParseException,
      DatatypeConfigurationException {
    String dateformatbefore = "2012-02-06T20:48:39.614+01:00";

    System.out.println("--------- Test Datestamp Parsing with a given date ---------------");
    System.out.println("Original date: " + dateformatbefore);
    XMLGregorianCalendar testDate = OAIPMHUtillities.convertDateFormat(dateformatbefore);
    System.out.println("Date after conversion: " + testDate);
    System.out.println("---------------------------------------------");
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  @Ignore
  public void testDateStringType() throws ParseException,
      DatatypeConfigurationException {
    String dateformatbefore = "2012-02-06T20:48:39.614+01:00";

    System.out.println("------------String Version-------------------");
    System.out.println("Original date: " + dateformatbefore);
    String testDate = OAIPMHUtillities.datestampAsString(dateformatbefore);
    System.out.println("Date after conversion: " + testDate);
    System.out.println("---------------------------------------------");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestIdentify() throws ParseException {

    System.out.println("Test for the verb \"Identify\" with succesfull response");
    JAXBElement<OAIPMHType> r = 
    		this.request.getRequest("Identify", "", "", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore  
  public void testGetRequestIdentifyVerbError() throws ParseException {
	System.out.println("Test for the verb \"Identify\" with error response (MetadaFormat)");
    JAXBElement<OAIPMHType> t = 
    		this.request.getRequest("Identify", "", "a", "", "", "", "");
    JAXB.marshal(t, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestIdentifyArgumentError() throws ParseException {
    System.out.println("Test for the verb \"Identify\" with error response (Until and Identifier)");
    JAXBElement<OAIPMHType> z = 
    		this.request.getRequest("Identify", "a", "", "", "", "a", "");
    JAXB.marshal(z, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordTextGrid() throws ParseException {

    record.setContributor(TGConstants.CONTRIBUTOR_LIST);
    record.setCoverage(TGConstants.COVERAGE_LIST);
    record.setCreator(TGConstants.CREATOR_LIST);
    record.setDates(TGConstants.DATE_LIST);
    record.setDescriptions(TGConstants.DESCRIPTION_LIST);
    record.setFormats(TGConstants.FORMAT_LIST);
    record.setIdentifiers(TGConstants.IDENTIFIER_LIST);
    record.setLanguages(TGConstants.LANGUAGE_LIST);
    record.setPublishers(TGConstants.PUBLISHER_LIST);
    record.setRelations(TGConstants.RELATIONS_LIST);
    record.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    record.setRights(TGConstants.RIGHTS_LIST);
    record.setSources(TGConstants.SOURCE_LIST);
    record.setSubjects(TGConstants.SUBJECT_LIST);
    record.setTitles(TGConstants.TITLE_LIST);
    record.setTypes(TGConstants.TYPE_LIST);
    record.setFields(TGConstants.TGFields);
    record.setFormatField(TGConstants.FORMAT);
    record.setFormatToFilter(TextGridMimetypes.EDITION);
    record.setDateOfObjectCreation(TGConstants.CREATED);
    record.setRelationToFurtherMetadataObject(TGConstants.EDITION_ISEDITIONOF);
    record.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    record.setIdentifierField("textgridUri");

    System.out.println("Test for the verb \"GetRecord\" with succesfull response");
    JAXBElement<OAIPMHType> p =
        this.request.getRequest("GetRecord", "textgrid:vqn0.0", "oai_dc", "", "", "", "");
    JAXB.marshal(p, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordIDIOM() throws ParseException {

    System.out.println("Test for the verb \"GetRecord\" with succesfull response");
    JAXBElement<OAIPMHType> p =
        this.request.getRequest("GetRecord", "textgrid:2rcj2", TGConstants.METADATA_IDIOM_PREFIX, "", "", "", "");
    JAXB.marshal(p, System.out);
    System.out.println("-----------------------------------\n");
  }
  
  
  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordErrorIdentifierMissing() throws ParseException {
    System.out
        .println("Test for the verb \"GetRecord\" with errpr response (Identifier Missing)");
    JAXBElement<OAIPMHType> p = this.request.getRequest("GetRecord", "",
        "", "", "", "", "");
    JAXB.marshal(p, System.out);
    System.out.println("-----------------------------------\n");

  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierSetsDC() throws ParseException {
	 
    OaiPmhTest.identifierList.setFieldForRange(TGConstants.RANGE_FIELD);
    OaiPmhTest.identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhTest.identifierList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTest.identifierList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTest.identifierList.setIdentifierField("textgridUri");
    OaiPmhTest.identifierList.setSearchResponseSize("100");
    System.out.println("Test for the verb \"ListIdentifiers\" with succesfull response");
    JAXBElement<OAIPMHType> p = this.request.getRequest("ListIdentifiers", "", "oai_dc",
        "project:TGPR-26236625-1acc-b921-a5fa-53567c3eeb80", "", "", "");
    JAXB.marshal(p, System.out);
    System.out.println("-----------------------------------\n");
  }
  
  @Test
  @Ignore
  public void testListIdentifierWithResumptionTokenDC() throws ParseException {
	 
    OaiPmhTest.identifierList.setFieldForRange(TGConstants.RANGE_FIELD);
    OaiPmhTest.identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhTest.identifierList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTest.identifierList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTest.identifierList.setIdentifierField("textgridUri");
    OaiPmhTest.identifierList.setSearchResponseSize("100");
    System.out.println("Test for the verb \"ListIdentifiers\" with succesfull response");
    JAXBElement<OAIPMHType> p = this.request.getRequest("ListIdentifiers", "", "oai_dc",
        "", "", "", "");
    JAXB.marshal(p, System.out);
    
    
    String resToken ="";
    for(Map.Entry<String, Integer> entry : IdentifierListDelivererDC.cursorCollector.entrySet()) {
    	resToken = entry.getKey();
    }   
    System.out.println("HASH MAP BEFORE: ");
    System.out.println(IdentifierListDelivererDC.cursorCollector);
    System.out.println(resToken);
    JAXBElement<OAIPMHType> p2 = this.request.getRequest("ListIdentifiers", "", "",
            "", "", "", resToken);
    JAXB.marshal(p2, System.out);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(IdentifierListDelivererDC.cursorCollector);
    
    System.out.println("-----------------------------------\n");
  }

  @Test
  @Ignore
  public void testListIdentifierIDIOMInvalidResumptionToken() throws ParseException {

    System.out.println("Test for the verb \"ListIdentifiers\" in IDIOM with an invalid resumption token");
    JAXBElement<OAIPMHType> p = this.request.getRequest("ListIdentifiers", "", "",
        "", "", "", "45d8b524-b69e-44e8-81a9-a3634005a430");
    JAXB.marshal(p, System.out);
    System.out.println("-----------------------------------\n");
  }
  
  @Test
  @Ignore
  public void testListIdentifierIDIOMWithResumptionToken() throws ParseException {
	
    System.out.println("Test for the verb \"ListIdentifiers\" in IDIOM with successfull response");
    
    JAXBElement<OAIPMHType> p = this.request.getRequest("ListIdentifiers", "", TGConstants.METADATA_IDIOM_PREFIX,
        "", "", "", "");
    JAXB.marshal(p, System.out);
    
    String resToken ="";
    for(Map.Entry<String, Integer> entry : IdentifierListDelivererIDIOM.cursorCollector.entrySet()) {
    	resToken = entry.getKey();
    }   
  
    JAXBElement<OAIPMHType> p2 = this.request.getRequest("ListIdentifiers", "", "",
            "", "", "", resToken);
    JAXB.marshal(p2, System.out);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(IdentifierListDelivererIDIOM.cursorCollector);
    
    System.out.println("-----------------------------------\n");

  }
  
  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecordSets() throws ParseException {
    OaiPmhTest.recordList.setContributor(TGConstants.CONTRIBUTOR_LIST);
    OaiPmhTest.recordList.setCoverage(TGConstants.COVERAGE_LIST);
    OaiPmhTest.recordList.setCreators(TGConstants.CREATOR_LIST);
    OaiPmhTest.recordList.setDates(TGConstants.DATE_LIST);
    OaiPmhTest.recordList.setDescriptions(TGConstants.DESCRIPTION_LIST);
    OaiPmhTest.recordList.setFormats(TGConstants.FORMAT_LIST);
    OaiPmhTest.recordList.setIdentifiers(TGConstants.IDENTIFIER_LIST);
    OaiPmhTest.recordList.setLanguages(TGConstants.LANGUAGE_LIST);
    OaiPmhTest.recordList.setPublishers(TGConstants.PUBLISHER_LIST);
    OaiPmhTest.recordList.setRelations(TGConstants.RELATIONS_LIST);
    OaiPmhTest.recordList.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    OaiPmhTest.recordList.setRights(TGConstants.RIGHTS_LIST);
    OaiPmhTest.recordList.setSources(TGConstants.SOURCE_LIST);
    OaiPmhTest.recordList.setSubjects(TGConstants.SUBJECT_LIST);
    OaiPmhTest.recordList.setTitles(TGConstants.TITLE_LIST);
    OaiPmhTest.recordList.setTypes(TGConstants.TYPE_LIST);
    OaiPmhTest.recordList.setFormatField(TGConstants.FORMAT);
    OaiPmhTest.recordList.setFormatToFilter(TextGridMimetypes.EDITION);
    OaiPmhTest.recordList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTest.recordList.setRelationToFurtherMetadataObject(TGConstants.EDITION_ISEDITIONOF);
    OaiPmhTest.recordList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTest.recordList.setFieldForRange(TGConstants.CREATED);
    OaiPmhTest.recordList.setModifiedField(TGConstants.MODIFIED_FIELD);
    OaiPmhTest.recordList.setIdentifierField("textgridUri");    
   
    System.out.println("Test for the verb \"ListRecords\" with sets with succesfull response");
    JAXBElement<OAIPMHType> p =
        this.request.getRequest("ListRecords", "", "oai_dc", "", "", "", "");
    JAXB.marshal(p, System.out);
    
    String resToken ="";
    for(Map.Entry<String, Integer> entry : RecordListDelivererDC.cursorCollector.entrySet()) {
    	resToken = entry.getKey();
    }   
  
    JAXBElement<OAIPMHType> p2 = this.request.getRequest("ListRecords", "", "",
            "", "", "", resToken);
    JAXB.marshal(p2, System.out);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(RecordListDelivererDC.cursorCollector);
    
    
    System.out.println("-----------------------------------\n");

  }

  @Test
  @Ignore
  public void testListRecordWithResumptionTokenForIDIOM() throws ParseException {
   
    System.out.println("Test for the verb \"ListRecords\" for IDIOM with succesfull response");
    JAXBElement<OAIPMHType> p =
        this.request.getRequest("ListRecords", "", TGConstants.METADATA_IDIOM_PREFIX, "", "", "", "");
    JAXB.marshal(p, System.out);
    
    
    String resToken ="";
    for(Map.Entry<String, Integer> entry : RecordListDelivererIDIOM.cursorCollector.entrySet()) {
    	resToken = entry.getKey();
    }   
  
    JAXBElement<OAIPMHType> p2 = this.request.getRequest("ListRecords", "", "",
            "", "", "", resToken);
    JAXB.marshal(p2, System.out);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(RecordListDelivererIDIOM.cursorCollector);
    
    System.out.println("-----------------------------------\n");

  }
  
  
  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecords() throws ParseException {

    JAXBElement<OAIPMHType> r =
        this.request.getRequest("ListRecords", "", "oai_dc", "", "2011-02-06T20:48:39", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecords2() throws ParseException {

    OaiPmhTest.recordList.setContributor(TGConstants.CONTRIBUTOR_LIST);
    OaiPmhTest.recordList.setCoverage(TGConstants.COVERAGE_LIST);
    OaiPmhTest.recordList.setCreators(TGConstants.CREATOR_LIST);
    OaiPmhTest.recordList.setDates(TGConstants.DATE_LIST);
    OaiPmhTest.recordList.setDescriptions(TGConstants.DESCRIPTION_LIST);
    OaiPmhTest.recordList.setFormats(TGConstants.FORMAT_LIST);
    OaiPmhTest.recordList.setIdentifiers(TGConstants.IDENTIFIER_LIST);
    OaiPmhTest.recordList.setLanguages(TGConstants.LANGUAGE_LIST);
    OaiPmhTest.recordList.setPublishers(TGConstants.PUBLISHER_LIST);
    OaiPmhTest.recordList.setRelations(TGConstants.RELATIONS_LIST);
    OaiPmhTest.recordList.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    OaiPmhTest.recordList.setRights(TGConstants.RIGHTS_LIST);
    OaiPmhTest.recordList.setSources(TGConstants.SOURCE_LIST);
    OaiPmhTest.recordList.setSubjects(TGConstants.SUBJECT_LIST);
    OaiPmhTest.recordList.setTitles(TGConstants.TITLE_LIST);
    OaiPmhTest.recordList.setTypes(TGConstants.TYPE_LIST);
    OaiPmhTest.recordList.setFormatField(TGConstants.FORMAT);
    OaiPmhTest.recordList.setFormatToFilter(TextGridMimetypes.EDITION);
    OaiPmhTest.recordList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTest.recordList.setRelationToFurtherMetadataObject(TGConstants.EDITION_ISEDITIONOF);
    OaiPmhTest.recordList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTest.recordList.setFieldForRange(TGConstants.CREATED);
    OaiPmhTest.recordList.setModifiedField(TGConstants.CREATED);
    OaiPmhTest.recordList.setIdentifierField(TGConstants.URI);
    OaiPmhTest.recordList.setSearchResponseSize("100");

    JAXBElement<OAIPMHType> r =
        this.request.getRequest("ListRecords", "", "oai_dc", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }



  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListMetadataFormatsForIdentifier() throws ParseException {
    JAXBElement<OAIPMHType> r = this.request.getRequest(
        "ListMetadataFormats", "7t.0", "", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListMetadataFormatsWithoutID() throws ParseException {
    JAXBElement<OAIPMHType> r = this.request.getRequest(
        "ListMetadataFormats", "", "", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListSets() throws ParseException {
    System.out.println("bla");
    this.setListTextGrid.setFormatField("format");
    this.setListTextGrid.setFormatToFilter(TextGridMimetypes.DARIAH_COLLECTION);
    this.setListTextGrid.setIdentifierField("textgridUri");
    this.setListTextGrid.setRepositoryObjectURIPrefix("textgrid:");
    JAXBElement<OAIPMHType> r = this.request.getRequest("ListSets", "", "", "", "", "", "");

    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }


  @Test
  @Ignore
  public void testEmptyRequest() throws ParseException {
    OaiPmhTest.identifierList.setFieldForRange(TGConstants.RANGE_FIELD);
    OaiPmhTest.identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhTest.identifierList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTest.identifierList.setIdentifierField("textgridUri");
    OaiPmhTest.identifierList.setRepositoryObjectURIPrefix("textgrid:");
    OaiPmhTest.identifierList.setSearchResponseSize("100");
    JAXBElement<OAIPMHType> r = this.request.getRequest("", "", "", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

}
